import mock
import unittest

from pastry.resources.nodes import Nodes


class NodesTestCase(unittest.TestCase):

    @mock.patch('pastry.resources.nodes.Base.exists', return_value=True)
    def test_exists(self, base):
        self.assertEqual(Nodes.exists('node'), True)

    @mock.patch('pastry.resources.nodes.Base.get_acl', return_value='acl')
    def test_get_acl(self, base):
        self.assertEqual(Nodes.get_acl('node'), 'acl')

    @mock.patch('pastry.resources.nodes.Base.set_permission', return_value={})
    def test_set_permission(self, base):
        self.assertEqual(Nodes.set_permission(
            'node',
            'permission',
            {}
        ), {})



'''users provides the methods for chef user resources'''

from .base import Base
from pastry.pastry_client import PastryClient


class Users(Base):
    '''
    Provides access to the chef `users` resource
    '''
    _base_url = '/users'

    @classmethod
    def index(cls):
        '''
        Fetches all of the users on the chef server

        :return: All the chef users with a url for each user
        :rtype: hash
        '''
        return super(Users, cls).index()

    @classmethod
    def get(cls, username):
        '''
        Fetches a user from the chef server

        :param username: The User's username
        :type username: string
        :return: The chef user
        :rtype: hash
        '''
        return super(Users, cls).get(username)

    @classmethod
    def create(cls, user):
        '''
        Creates a new chef user on the server

        The user hash should be in the form::

            {
                'username': <username>,
                'display_name': <display_name>,
                'first_name': <first name>,
                'middle_name': <middle name>, # optional
                'last_name': <last name>,
                'email': <email>,
                'password': <password>
            }

        :param user: The user to create
        :type user: hash
        :return: The username and url
        :rtype: hash
        '''
        return super(Users, cls).create(user)

    @classmethod
    def update(cls, username, user):
        '''
        Updates a user on the chef server

        :param username: The User's username
        :param user: The User members and content
        :type username: string
        :type user: hash
        :return: The username and url
        :rtype: hash
        '''
        return super(Users, cls).update(username, user)

    @classmethod
    def delete(cls, username):
        '''
        Deletes a user from the chef server

        :param username: The User's username
        :type usename: string
        :return: The deleted user's name
        :rtype: hash
        '''
        return super(Users, cls).delete(username)

    @classmethod
    def exists(cls, username):
        '''
        Checks if a user exists on the chef server

        :param username: The User's username
        :type usename: string
        :return: If the user exists
        :rtype: boolean
        '''
        return super(Users, cls).exists(username)

    @classmethod
    def invite(cls, username, orgname):
        '''
        Invite a user to an org

        :param username: The User's username
        :param orgname: The chef organization
        :type username: string
        :type orgname: string
        :return: If the request was successful
        :rtype: boolean
        '''
        return PastryClient.call(
            'organizations/%s/association_requests' % orgname,
            method='POST',
            data={'user': username}
        )



import mock
import unittest

from pastry.resources.pushy import Pushy


class PushyTestCase(unittest.TestCase):

    @mock.patch('pastry.resources.pushy.Pushy.base_url', return_value='base')
    @mock.patch('pastry.resources.pushy.PastryClient.call', return_value={})
    def test_exists(self, call, base_url):
        self.assertEqual(Pushy.status('node'), {})
        call.assert_called_with('base/node_states/node')



'''Groups provides the methods for chef group resources'''

from .base import Base


class Groups(Base):
    '''
    Provides access to the chef `groups` resource
    '''
    _base_url = '/organizations/%(org)s/groups'

    @classmethod
    def index(cls):
        '''
        Fetches all of the groups on the chef server

        :return: All the chef groups with a url for each group
        :rtype: hash
        '''
        return super(Groups, cls).index()

    @classmethod
    def get(cls, groupname):
        '''
        Fetches a group from the chef server

        .. note::

            Chef returns a slightly different format to
            what it expects for create/update groups.

        :param groupname: The Group's name
        :type groupname: string
        :return: The chef group and members
        :rtype: hash
        '''
        return super(Groups, cls).get(groupname)

    @classmethod
    def create(cls, group):
        '''
        Creates a new chef group on the server

        The group hash should be in the form::

            {
                'groupname': <groupname>,
                'name': <groupname>,
                'actors': { #optional
                    'users': <list of usernames>, # optional
                    'clients': <list of clients>, #optional
                    'groups': <list of groups> #optional
                }
            }

        :param group: The group to create
        :type group: hash
        :return: The groupname and url
        :rtype: hash
        '''
        return super(Groups, cls).create(group)

    @classmethod
    def update(cls, groupname, group):
        '''
        Updates a group on the chef server

        the group hash should be in the same format as for create

        :param groupname: The Group's groupname
        :param group: The Group members and content
        :type groupname: string
        :type group: hash
        :return: The groupname and url
        :rtype: hash
        '''
        return super(Groups, cls).update(groupname, group)

    @classmethod
    def delete(cls, groupname):
        '''
        Deletes a group from the chef server

        :param groupname: The Group's groupname
        :type groupname: string
        :return: The deleted group's name
        :rtype: hash
        '''
        return super(Groups, cls).delete(groupname)

    @classmethod
    def exists(cls, groupname):
        '''
        Checks if a group exists on the chef server

        :param username: The Group's groupname
        :type usename: string
        :return: If the group exists
        :rtype: boolean
        '''
        return super(Groups, cls).exists(groupname)



import mock
import unittest

from pastry.resources.users import Users


class UsersTestCase(unittest.TestCase):

    @mock.patch('pastry.resources.users.Base.index', return_value='index')
    def test_index(self, base):
        self.assertEqual(Users.index(), 'index')

    @mock.patch('pastry.resources.users.Base.get', return_value='user')
    def test_get(self, base):
        self.assertEqual(Users.get('username'), 'user')

    @mock.patch('pastry.resources.users.Base.create', return_value='user')
    def test_create(self, base):
        self.assertEqual(Users.create({}), 'user')

    @mock.patch('pastry.resources.users.Base.update', return_value='user')
    def test_update(self, base):
        self.assertEqual(Users.update('user', {}), 'user')

    @mock.patch('pastry.resources.users.Base.delete', return_value='user')
    def test_delete(self, base):
        self.assertEqual(Users.delete('user'), 'user')

    @mock.patch('pastry.resources.users.Base.exists', return_value=True)
    def test_exists(self, base):
        self.assertEqual(Users.exists('user'), True)

    @mock.patch('pastry.resources.users.PastryClient.call', return_value=True)
    def test_invite(self, call):
        self.assertEqual(Users.invite('user', 'org'), True)
        call.assert_called_with(
            'organizations/org/association_requests',
            method='POST',
            data={'user': 'user'}
        )



import os

try:
    from setuptools import setup, find_packages
    from setuptools.command.test import test as TestCommand
except ImportError:
    from ez_setup import use_setuptools
    use_setuptools()
    from setuptools import setup, find_packages
    from setuptools.command.test import test as TestCommand

class PyTest(TestCommand):
    user_options = [('pytest-args=', 'a', "Arguments to pass to pytest")]

    def initialize_options(self):
        TestCommand.initialize_options(self)
        self.pytest_args = ''

    def run_tests(self):
        #import here, cause outside the eggs aren't loaded
        import sys
        import pytest
        errno = pytest.main(self.pytest_args.split(' '))
        sys.exit(errno)

install_requires = [
    'rsa',
    'pyyaml',
    'requests'
]

tests_require = [
    'mock',
    'pytest-cov',
    'coverage',
    'pytest'
]

setup(
    name='pastry',
    version='0.1.18',
    description='A simple api wrapper for chef',
    long_description=open(os.path.join(os.path.dirname(__file__), 'README.rst')).read(),
    author='Stephen Breyer-Menke',
    author_email='steve.bm@gmail.com',
    license='MIT',
    url='https://github.com/stephenbm/pastry',
    packages=find_packages(),
    test_suite='tests',
    tests_require=tests_require,
    cmdclass={'test': PyTest},
    install_requires=install_requires,
    include_package_data=True,
    zip_safe=False
)



'''environments provides the methods for chef environment resources'''

from .base import Base


class Environments(Base):
    '''
    Provides access to the chef `environments` resource
    '''
    _base_url = '/organizations/%(org)s/environments'

    @classmethod
    def index(cls):
        '''
        Fetches all of the environments on the chef server

        :return: All the chef environments with a url for each environment
        :rtype: hash
        '''
        return super(Environments, cls).index()

    @classmethod
    def get(cls, environmentname):
        '''
        Fetches a environment from the chef server

        :param environmentname: The Environment's environmentname
        :type environmentname: string
        :return: The chef environment
        :rtype: hash
        '''
        return super(Environments, cls).get(environmentname)

    @classmethod
    def create(cls, environment):
        '''
        Creates a new chef environment on the server

        :param environment: The environment to create
        :type environment: hash
        :return: The environmentname and url
        :rtype: hash
        '''
        return super(Environments, cls).create(environment)

    @classmethod
    def update(cls, environmentname, environment):
        '''
        Updates a environment on the chef server

        :param environmentname: The Environment's name
        :param environment: The Environment's config
        :type environmentname: string
        :type environment: hash
        :return: The environment name and url
        :rtype: hash
        '''
        return super(Environments, cls).update(environmentname, environment)

    @classmethod
    def delete(cls, environmentname):
        '''
        Deletes a environment from the chef server

        :param environmentname: The Environment's environmentname
        :type usename: string
        :return: The deleted environment's name
        :rtype: hash
        '''
        return super(Environments, cls).delete(environmentname)

    @classmethod
    def exists(cls, environmentname):
        '''
        Checks if a environment exists on the chef server

        :param environmentname: The Environment's environmentname
        :type usename: string
        :return: If the environment exists
        :rtype: boolean
        '''
        return super(Environments, cls).exists(environmentname)
